/*============================================================================

                        PVK40 Example : Serial EEPROM 24xx (SW)

 Compiler/IDE : PICC-Lite and MPLAB

 Processor : PIC16F877A
 Clock : XT 3.2768 MHz

 On your PVK40 board set DIP switches according to following way:
 1) Insert 24LC01B into 24xx socket
 2) Switch off all DIP switches except:
 3) Switch on OSC 3.276M on S9
 4) Switch on B0,1,2,4 DISP on S11
 5) Switch on B3 LED on S11

============================================================================*/


/*============================================================================

       INCLUDE

============================================================================*/
#include <pic.h>


/*============================================================================

       Configuration Bits

============================================================================*/
__CONFIG(UNPROTECT & DEBUGDIS & WRTEN & DUNPROT & LVPDIS 
   & BOREN & PWRTEN & WDTDIS & XT); 


/*============================================================================

       User definition

============================================================================*/
#define PORTBIT(adr,bit) ((unsigned)(&adr)*8+(bit)) //no modify this line

typedef unsigned int clock_t;

static bit led @ PORTBIT(PORTB,3);
static bit pushbutton0 @ PORTBIT(PORTD,0);
static bit i2c_sda @ PORTBIT(PORTC,4);
static bit i2c_scl @ PORTBIT(PORTC,3);
static bit i2c_sda_tris @ PORTBIT(TRISC,4);
static bit i2c_scl_tris @ PORTBIT(TRISC,3);

char digit[5] = {0xFF,0xFE,0xFD,0xFB,0xEF}; 
char segment[10] = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90};

clock_t systimer;         /* systimer is incremented every 20ms */
clock_t now;
clock_t lap;
unsigned int period;
unsigned int count;

/*============================================================================

       Program

============================================================================*/

void interrupt timer0(void)
{
  T0IF=0;
  systimer++;
}

clock_t clock(void)        /* read systimer */
{
  clock_t temp;

  GIE=0;
  temp=systimer;
  GIE=1;
  return temp;
}

/* dispdigit(j,p) - shows the value of an integer j
   on one p of the 7 segments display            */ 
void dispdigit(int j, int p)
{
  PORTB=(digit[0]&0xF7)|(PORTB&0x08);
  PORTD=segment[j];
  TRISD=0x00;
  PORTB=(digit[p]&0xF7)|(PORTB&0x08);
}

void timer0init(void)
{    /* initialisation of timer0 */
  PS2=1;  PS1=0;  PS0 = 1; /* select prescaler by 64 */
  T0CS = 0;                /* select timer0 */
  PSA=0;                   /* select prescaler for timer0 */
  TMR0 =0;                 /* preload timer0 */
  T0IE = 1;                /* interrupts are actived */
}

void portinit(void)
{
  PORTB=0xF7;
  PORTC=0x00;
  PORTD=0xFF;
  TRISB=0xE0;
  TRISC=0xFF;
  TRISD=0x00;
}

void delayus(unsigned char us)
{
  for (;us--;);
}

void i2c_start_bit(void)
{
  i2c_sda_tris=1;  /* set data bus */
  delayus(1);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
}

void i2c_stop_bit(void)
{
  i2c_scl_tris=0;  /* reset clock bus */
  delayus(1);
  i2c_sda_tris=0;  /* reset data bus */
  delayus(2);
  i2c_scl_tris=1;  /* set clock bus */
  delayus(2);
  i2c_sda_tris=1;  /* set data bus */
}

unsigned char i2c_write(unsigned char data)
/* send byte to i2c device 
   returned value is acknowledge flag */
{ 
  unsigned char mask, flag;

  for(mask=0x80;mask!=0;mask>>=1) {
    if(data&mask)
      i2c_sda_tris=1;
    else
      i2c_sda_tris=0;
    i2c_scl_tris=1;  /* set clock bus */
    asm("nop");
    i2c_scl_tris=0;  /* reset clock bus */
    asm("nop");
  }
  i2c_sda_tris=1;  /* set data bus */
  i2c_scl_tris=1;  /* set clock bus - 9. clock pulse */
  asm("nop");
  flag=i2c_sda; /* read ACKF */
  i2c_scl_tris=0;  /* reset clock bus */
  return flag;
}

unsigned char i2c_read(unsigned char ackdata)
{  /* read byte from i2c bus */
  unsigned char temp, bitcnt;

  i2c_sda_tris=1;  /* set data bus (Hi-Impedance) */
  temp=0x00;
  for(bitcnt=8;bitcnt!=0;bitcnt--) {
    i2c_scl_tris=1;  /* set clock bus */
    temp<<=1;
    temp|=i2c_sda; /* read data */
    asm("nop");
    i2c_scl_tris=0;  /* reset clock bus */
    asm("nop");
  }
  if(ackdata) 
    i2c_sda_tris=1;
  else
    i2c_sda_tris=0;
  i2c_scl_tris=1;  /* set clock bus - send ACK */
  delayus(2);
  i2c_scl_tris=0;  /* reset clock bus */
  asm("nop");
  return temp;
}

void eep_wrt(unsigned char address, unsigned char data)
{
  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(!i2c_write(0xA0))  /* control byte - write */
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write(address); /* write address to EEPROM */
  i2c_write(data);  /* write data to EEPROM */
  i2c_stop_bit(); /* stop bit */
}

unsigned char eep_read(unsigned char address)
{
  unsigned char tempdata;

  for(;;) {     /* acknowledge pooling loop */
    i2c_start_bit();  /* start bit */
    if(!i2c_write(0xA0))  /* control byte - write */
      break;
    i2c_stop_bit();   /* stop bit */
  }
  i2c_write(address); /* write address to EEPROM */
  i2c_start_bit();
  i2c_write(0xA1);  /* control byte - read */
  tempdata=i2c_read(1);  /* select NO_ACK bit */
  i2c_stop_bit(); /* stop bit */
  return tempdata;
}

void eep_write(unsigned char address, unsigned char data)
{
  if (eep_read(address)!=data)
    eep_wrt(address,data);
}

void main(void)
{
  portinit();
  timer0init();
  period=50;               /* 1 second */
  GIE=1;                   /* interrupts are actived */

  count=eep_read(0x00)+(eep_read(0x01)<<8);

  for(;;) {
    now=clock();
    if ((now-lap)>period) {
      lap+=period;
      if(++count>9999)     /*counting*/
        count=0;
    }
    dispdigit(count%10,1);
    delayus(250);
    dispdigit((count/10)%10,2);
    delayus(250);
    dispdigit((count/100)%10,3);
    delayus(250);
    dispdigit(count/1000,4);
    delayus(250);

    TRISD=0xFF;   /* inputs */
    PORTB=digit[0]&0xF7; /* mask LED output */
    delayus(5);
    if(!pushbutton0) {
      led=1;
      eep_write(0x00,count&0xFF);
      eep_write(0x01,count>>8);
    }
  } // end of the for loop
}
